@@ -1,13 +1,9 @@
 package Module::Pluggable::Singleton;
-{
-  $Module::Pluggable::Singleton::DIST = 'Module-Pluggable-Singleton';
-}
 
 use strict;
 use warnings;
-use Module::Pluggable::Singleton::Object;
+use Module::Pluggable::Object;
 use Carp;
-use Data::Dump qw/pp/;
 
 =head1 NAME
 
@@ -15,24 +11,17 @@ Module::Pluggable::Singleton - call/return single plugins on demand using shorte
 
 =head1 VERSION
 
-version 0.03.0
+Version 0.01
 
 =cut
 
-our $VERSION = '0.03.0';
+our $VERSION = '0.01';
 
 =head1 SYNOPSIS
 
     package MyPluggable;
     use Module::Pluggable::Singleton;
 
-    my $plugins = MyPluggable->plugin;
-    my @plugins = MyPluggable->plugin;
-
-    my @plugins = MyPluggable->plugins;
-
-    my $plugin_long = $logic->plugin($name);
-
     my $plugins = MyPluggable->plugin;
     my $nick = MyPluggable->call('Robert','nickname',$person->full_name);
 
@@ -42,7 +31,7 @@ our $VERSION = '0.03.0';
 
 =head2 find($name)
 
-Return the instance of the plugin with given short name
+Return the instance of the plugin with the given name
 
     $plugin = $logic->find('Bar');
 
@@ -54,10 +43,6 @@ module when a short name is given
     my @plugin_short = $logic->plugin
     my $plugin_long = $logic->plugin($name);
 
-=head2 plugin($name)
-
-Return an array of all the module names
-
 =head2 call($plugin_name, $method, $params)
 
 Using the plugin $plugin_name call $method with $params
@@ -69,47 +54,98 @@ Using the plugin $plugin_name call $method with $params
 sub import {
     my ($class, %opts) = @_;
     my $caller = (caller)[0];
-    $opts{require} = 1; # you find out earlier if it has a syntax error
-    $opts{package} = $caller;
+    require Module::Pluggable;
 
-    my $finder = Module::Pluggable::Singleton::Object->new(%opts);
+    $opts{search_path} = "${caller}::Plugin";
+    Module::Pluggable->import(
+        package     => $caller,
+        search_path => $opts{search_path},
+        require => 1
+    );
 
-    if (!$opts{search_path}) {
-        $opts{search_path} = "${caller}::Plugin";
-    }
-    if ($opts{search_path}) {
-        if (ref($opts{search_path}) eq '') {
-            $opts{search_path} = [$opts{search_path}];
+    my $namespace  = "${caller}::Plugin";
+    my $base_class = "${caller}::Base";
+    my $to_chop    = "${namespace}";
+    my $plugin_for = { };
+    my $instance_of = { };
+    foreach my $plugin ($caller->plugins) {
+        my $shortname = $plugin;
+        $shortname =~ s/^${to_chop}:://;
+
+# FIXME:
+#        if (not $plugin->isa($base_class)) {
+#            confess __PACKAGE__ .": plugin '$shortname' needs to implement "
+#                ."'". $base_class ."'";
+#        }
+
+        if (exists $plugin_for->{$shortname}) {
+            confess __PACKAGE__ .": Plugin already exists for '$shortname'";
         }
-    }
 
-    my $namespace  = "${caller}::". ucfirst($opts{sub_name} || 'plugins');
-    my $sub_name = $opts{sub_name} || 'plugins';
-    my $plugin_for = { }; # maps shortname to module name
-    my $instance_of = { }; # instances
+        $plugin_for->{$shortname} = $plugin;
+    }
 
     my $find_sub = sub {
-        shift @_;
-        return $finder->find(@_);
+        my($self,$label) = @_;
+
+        if (!defined $label) {
+            warn __PACKAGE__ .": Not provided name of plugin";
+            return;
+        }
+
+        my $name = $plugin_for->{$label} || undef;
+        if (!defined $name) {
+            die __PACKAGE__ .": Not possible to load module '$label'";
+        }
+
+
+        # use an existing instance or create a new one.. and keep ref to it
+        my $instance = $instance_of->{$label}
+            || (defined $name ? $name->new() : undef);
+
+        if ($instance && not defined $instance_of->{$label}) {
+            $instance_of->{$label} = $instance;
+        }
+
+        return $instance;
     };
 
     my $plugin_sub = sub {
-        shift @_;
-        return $finder->plugin(@_);
-    };
-    my $call_sub = sub {
-        shift @_;
-        return $finder->call(@_);
+        my($self,$name) = @_;
+
+        return keys %{$plugin_for} if (!defined $name);
+        
+        return defined $plugin_for->{$name}
+            ? $plugin_for->{$name} : undef;
     };
 
-    my $plugins_sub = sub {
-        my($self) = @_;
-        $finder->plugins(@_);
+    my $call_sub = sub {
+        my $self = shift @_;
+        my $label = shift @_;
+        my $method = shift @_;
+
+        if (!defined $label) {
+            die __PACKAGE__ .": Plugin name not provided";
+            return;
+        }
+
+        if (!defined $method) {
+            die __PACKAGE__ .": Method name not provided";
+            return;
+        }
+
+
+        my $instance = $self->find($label);
+        if (!$instance->can($method)) {
+            die __PACKAGE__ .": Cannot call '$method' on '$label' plugin";
+        }
+
+        return $instance->$method(@_);
     };
+
     no strict 'refs';
     no warnings qw(redefine prototype);
 
-    *{"$caller\::$sub_name"} = $plugins_sub;
     *{"$caller\::find"} = $find_sub;
     *{"$caller\::plugin"} = $plugin_sub;
     *{"$caller\::call"} = $call_sub;
@@ -175,3 +211,4 @@ See http://dev.perl.org/licenses/ for more information.
 =cut
 
 1; # End of Module::Pluggable::Singleton
+
